//
// Copyright(C) 2021 by Ryan Krafnick
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// DESCRIPTION:
//	DSDA Line Special
//

#include <stddef.h>

#include "line_special.h"

static zl_linespecial_t zl_linespecial_spec[zl_linespecial_max] = {
  // special name, special number, min script args, max script args, num args on a line
  [1] =
  { "Polyobj_StartLine", 1, -1, -1, 4 },
  { "Polyobj_RotateLeft", 2, 3, 3, 3 },
  { "Polyobj_RotateRight", 3, 3, 3, 3 },
  { "Polyobj_Move", 4, 4, 4, 4 },
  { "Polyobj_ExplicitLine", 5, -1, -1, 5 },
  { "Polyobj_MoveTimes8", 6, 4, 4, 4 },
  { "Polyobj_DoorSwing", 7, 4, 4, 4 },
  { "Polyobj_DoorSlide", 8, 5, 5, 5 },
  { "Line_Horizon", 9, -1, 0, 0 },
  { "Door_Close", 10, 2, 3, 3 },
  { "Door_Open", 11, 2, 3, 3 },
  { "Door_Raise", 12, 3, 4, 4 },
  { "Door_LockedRaise", 13, 4, 5, 5 },
  { "Door_Animated", 14, 4, 4, 4 },
  { "Autosave", 15, 0, 0, 0 },
  { "Transfer_WallLight", 16, -1, -1, 2 },
  { "Thing_Raise", 17, 1, 1, 1 },
  { "StartConversation", 18, 1, 2, 2 },
  { "Thing_Stop", 19, 1, 1, 1 },
  { "Floor_LowerByValue", 20, 3, 4, 4 },
  { "Floor_LowerToLowest", 21, 2, 3, 3 },
  { "Floor_LowerToNearest", 22, 2, 3, 3 },
  { "Floor_RaiseByValue", 23, 3, 5, 5 },
  { "Floor_RaiseToHighest", 24, 2, 5, 5 },
  { "Floor_RaiseToNearest", 25, 2, 4, 4 },
  { "Stairs_BuildDown", 26, 5, 5, 5 },
  { "Stairs_BuildUp", 27, 5, 5, 5 },
  { "Floor_RaiseAndCrush", 28, 3, 4, 4 },
  { "Pillar_Build", 29, 3, 3, 3 },
  { "Pillar_Open", 30, 4, 4, 4 },
  { "Stairs_BuildDownSync", 31, 4, 4, 4 },
  { "Stairs_BuildUpSync", 32, 4, 4, 4 },
  { "ForceField", 33, 0, 0, 0 },
  { "ClearForceField", 34, 1, 1, 1 },
  { "Floor_RaiseByValueTimes8", 35, 3, 5, 5 },
  { "Floor_LowerByValueTimes8", 36, 3, 4, 4 },
  { "Floor_MoveToValue", 37, 3, 4, 5 },
  { "Ceiling_Waggle", 38, 5, 5, 5 },
  { "Teleport_ZombieChanger", 39, 2, 2, 2 },
  { "Ceiling_LowerByValue", 40, 3, 5, 5 },
  { "Ceiling_RaiseByValue", 41, 3, 4, 4 },
  { "Ceiling_CrushAndRaise", 42, 3, 4, 4 },
  { "Ceiling_LowerAndCrush", 43, 3, 4, 4 },
  { "Ceiling_CrushStop", 44, 1, 2, 2 },
  { "Ceiling_CrushRaiseAndStay", 45, 3, 4, 4 },
  { "Floor_CrushStop", 46, 1, 1, 1 },
  { "Ceiling_MoveToValue", 47, 3, 5, 5 },
  { "Sector_Attach3dMidtex", 48, -1, -1, 3 },
  { "GlassBreak", 49, 0, 1, 1 },
  { "ExtraFloor_LightOnly", 50, -1, -1, 2 },
  { "Sector_SetLink", 51, 4, 4, 4 },
  { "Scroll_Wall", 52, 5, 5, 5 },
  { "Line_SetTextureOffset", 53, 5, 5, 5 },
  { "Sector_ChangeFlags", 54, 3, 3, 3 },
  { "Line_SetBlocking", 55, 3, 3, 3 },
  { "Line_SetTextureScale", 56, 5, 5, 5 },
  { "Sector_SetPortal", 57, -1, -1, 5 },
  { "Sector_CopyScroller", 58, -1, -1, 2 },
  { "Polyobj_OR_MoveToSpot", 59, 3, 3, 3 },
  { "Plat_PerpetualRaise", 60, 3, 3, 3 },
  { "Plat_Stop", 61, 1, 2, 2 },
  { "Plat_DownWaitUpStay", 62, 3, 3, 3 },
  { "Plat_DownByValue", 63, 4, 4, 4 },
  { "Plat_UpWaitDownStay", 64, 3, 3, 3 },
  { "Plat_UpByValue", 65, 4, 4, 4 },
  { "Floor_LowerInstant", 66, 3, 4, 4 },
  { "Floor_RaiseInstant", 67, 3, 5, 5 },
  { "Floor_MoveToValueTimes8", 68, 4, 5, 5 },
  { "Ceiling_MoveToValueTimes8", 69, 4, 5, 5 },
  { "Teleport", 70, 1, 3, 3 },
  { "Teleport_NoFog", 71, 1, 4, 4 },
  { "ThrustThing", 72, 2, 4, 4 },
  { "DamageThing", 73, 1, 2, 2 },
  { "Teleport_NewMap", 74, 2, 3, 3 },
  { "Teleport_EndGame", 75, 0, 0, 0 },
  { "TeleportOther", 76, 3, 3, 3 },
  { "TeleportGroup", 77, 5, 5, 5 },
  { "TeleportInSector", 78, 4, 5, 5 },
  { "Thing_SetConversation", 79, 2, 2, 2 },
  { "ACS_Execute", 80, 1, 5, 5 },
  { "ACS_Suspend", 81, 2, 2, 2 },
  { "ACS_Terminate", 82, 2, 2, 2 },
  { "ACS_LockedExecute", 83, 5, 5, 5 },
  { "ACS_ExecuteWithResult", 84, 1, 5, 5 },
  { "ACS_LockedExecuteDoor", 85, 5, 5, 5 },
  { "Polyobj_MoveToSpot", 86, 3, 3, 3 },
  { "Polyobj_Stop", 87, 1, 1, 1 },
  { "Polyobj_MoveTo", 88, 4, 4, 4 },
  { "Polyobj_OR_MoveTo", 89, 4, 4, 4 },
  { "Polyobj_OR_RotateLeft", 90, 3, 3, 3 },
  { "Polyobj_OR_RotateRight", 91, 3, 3, 3 },
  { "Polyobj_OR_Move", 92, 4, 4, 4 },
  { "Polyobj_OR_MoveTimes8", 93, 4, 4, 4 },
  { "Pillar_BuildAndCrush", 94, 4, 5, 5 },
  { "FloorAndCeiling_LowerByValue", 95, 3, 3, 3 },
  { "FloorAndCeiling_RaiseByValue", 96, 3, 3, 3 },
  { "Ceiling_LowerAndCrushDist", 97, 3, 5, 5 },
  { "Sector_SetTranslucent", 98, 3, 4, 4 },
  { "Floor_RaiseAndCrushDoom", 99, 3, 4, 4 },
  { "Scroll_Texture_Left", 100, -1, -1, 2 },
  { "Scroll_Texture_Right", 101, -1, -1, 2 },
  { "Scroll_Texture_Up", 102, -1, -1, 2 },
  { "Scroll_Texture_Down", 103, -1, -1, 2 },
  { "Ceiling_CrushAndRaiseSilentDist", 104, 3, 5, 5 },
  { "Door_WaitRaise", 105, 4, 5, 5 },
  { "Door_WaitClose", 106, 4, 5, 5 },
  { "Line_SetPortalTarget", 107, 2, 2, 2 },

  [109] =
  { "Light_ForceLightning", 109, 1, 1, 1 },
  { "Light_RaiseByValue", 110, 2, 2, 2 },
  { "Light_LowerByValue", 111, 2, 2, 2 },
  { "Light_ChangeToValue", 112, 2, 2, 2 },
  { "Light_Fade", 113, 3, 3, 3 },
  { "Light_Glow", 114, 4, 4, 4 },
  { "Light_Flicker", 115, 3, 3, 3 },
  { "Light_Strobe", 116, 5, 5, 5 },
  { "Light_Stop", 117, 1, 1, 1 },
  { "Plane_Copy", 118, -1, -1, 5 },
  { "Thing_Damage", 119, 2, 3, 3 },
  { "Radius_Quake", 120, 5, 5, 5 },  // Earthquake
  { "Line_SetIdentification", 121, -1, -1, 5 },

  [125] =
  { "Thing_Move", 125, 2, 3, 3 },

  [127] =
  { "Thing_SetSpecial", 127, 5, 5, 5 },
  { "ThrustThingZ", 128, 4, 4, 4 },
  { "UsePuzzleItem", 129, 2, 5, 5 },
  { "Thing_Activate", 130, 1, 1, 1 },
  { "Thing_Deactivate", 131, 1, 1, 1 },
  { "Thing_Remove", 132, 1, 1, 1 },
  { "Thing_Destroy", 133, 1, 3, 3 },
  { "Thing_Projectile", 134, 5, 5, 5 },
  { "Thing_Spawn", 135, 3, 4, 4 },
  { "Thing_ProjectileGravity", 136, 5, 5, 5 },
  { "Thing_SpawnNoFog", 137, 3, 4, 4 },
  { "Floor_Waggle", 138, 5, 5, 5 },
  { "Thing_SpawnFacing", 139, 2, 4, 4 },
  { "Sector_ChangeSound", 140, 2, 2, 2 },

  [154] =
  { "Teleport_NoStop", 154, 2, 3, 3 },

  [156] =
  { "Line_SetPortal", 156, -1, -1, 4 },
  { "SetGlobalFogParameter", 157, 2, 2, 2 },
  { "FS_Execute", 158, 1, 4, 4 },
  { "Sector_SetPlaneReflection", 159, 3, 3, 3 },
  { "Sector_Set3DFloor", 160, -1, -1, 5 },
  { "Sector_SetContents", 161, -1, -1, 3 },

  [168] =
  { "Ceiling_CrushAndRaiseDist", 168, 3, 5, 5 },
  { "Generic_Crusher2", 169, 5, 5, 5 },
  { "Sector_SetCeilingScale2", 170, 3, 3, 3 },
  { "Sector_SetFloorScale2", 171, 3, 3, 3 },
  { "Plat_UpNearestWaitDownStay", 172, 3, 3, 3 },
  { "NoiseAlert", 173, 2, 2, 2 },
  { "SendToCommunicator", 174, 4, 4, 4 },
  { "Thing_ProjectileIntercept", 175, 5, 5, 5 },
  { "Thing_ChangeTID", 176, 2, 2, 2 },
  { "Thing_Hate", 177, 2, 3, 3 },
  { "Thing_ProjectileAimed", 178, 4, 5, 5 },
  { "ChangeSkill", 179, 1, 1, 1 },
  { "Thing_SetTranslation", 180, 2, 2, 2 },
  { "Plane_Align", 181, -1, -1, 3 },
  { "Line_Mirror", 182, -1, 0, 0 },
  { "Line_AlignCeiling", 183, 2, 2, 2 },
  { "Line_AlignFloor", 184, 2, 2, 2 },
  { "Sector_SetRotation", 185, 3, 3, 3 },
  { "Sector_SetCeilingPanning", 186, 5, 5, 5 },
  { "Sector_SetFloorPanning", 187, 5, 5, 5 },
  { "Sector_SetCeilingScale", 188, 5, 5, 5 },
  { "Sector_SetFloorScale", 189, 5, 5, 5 },
  { "Static_Init", 190, -1, -1, 4 },
  { "SetPlayerProperty", 191, 3, 3, 3 },
  { "Ceiling_LowerToHighestFloor", 192, 2, 4, 5 },
  { "Ceiling_LowerInstant", 193, 3, 5, 5 },
  { "Ceiling_RaiseInstant", 194, 3, 4, 4 },
  { "Ceiling_CrushRaiseAndStayA", 195, 4, 5, 5 },
  { "Ceiling_CrushAndRaiseA", 196, 4, 5, 5 },
  { "Ceiling_CrushAndRaiseSilentA", 197, 4, 5, 5 },
  { "Ceiling_RaiseByValueTimes8", 198, 3, 4, 4 },
  { "Ceiling_LowerByValueTimes8", 199, 3, 5, 5 },
  { "Generic_Floor", 200, 5, 5, 5 },
  { "Generic_Ceiling", 201, 5, 5, 5 },
  { "Generic_Door", 202, 5, 5, 5 },
  { "Generic_Lift", 203, 5, 5, 5 },
  { "Generic_Stairs", 204, 5, 5, 5 },
  { "Generic_Crusher", 205, 5, 5, 5 },
  { "Plat_DownWaitUpStayLip", 206, 4, 5, 5 },
  { "Plat_PerpetualRaiseLip", 207, 4, 4, 4 },
  { "TranslucentLine", 208, 2, 3, 3 },
  { "Transfer_Heights", 209, -1, -1, 2 },
  { "Transfer_FloorLight", 210, -1, -1, 1 },
  { "Transfer_CeilingLight", 211, -1, -1, 1 },
  { "Sector_SetColor", 212, 4, 5, 5 },
  { "Sector_SetFade", 213,  4, 4, 4 },
  { "Sector_SetDamage", 214, 3, 5, 5 },
  { "Teleport_Line", 215, 2, 3, 3 },
  { "Sector_SetGravity", 216, 3, 3, 3 },
  { "Stairs_BuildUpDoom", 217, 5, 5, 5 },
  { "Sector_SetWind", 218, 4, 4, 4 },
  { "Sector_SetFriction", 219, 2, 2, 2 },
  { "Sector_SetCurrent", 220, 4, 4, 4 },
  { "Scroll_Texture_Both", 221, 5, 5, 5 },
  { "Scroll_Texture_Model", 222, -1, -1, 2 },
  { "Scroll_Floor", 223, 4, 4, 5 },
  { "Scroll_Ceiling", 224, 4, 4, 5 },
  { "Scroll_Texture_Offsets", 225, -1, -1, 1 },
  { "ACS_ExecuteAlways", 226, 1, 5, 5 },
  { "PointPush_SetForce", 227, -1, -1, 4 },
  { "Plat_RaiseAndStayTx0", 228, 2, 3, 3 },
  { "Thing_SetGoal", 229, 3, 4, 4 },
  { "Plat_UpByValueStayTx", 230, 3, 3, 3 },
  { "Plat_ToggleCeiling", 231, 1, 1, 1 },
  { "Light_StrobeDoom", 232, 3, 3, 3 },
  { "Light_MinNeighbor", 233, 1, 1, 1 },
  { "Light_MaxNeighbor", 234, 1, 1, 1 },
  { "Floor_TransferTrigger", 235, 1, 1, 1 },
  { "Floor_TransferNumeric", 236, 1, 1, 1 },
  { "ChangeCamera", 237, 3, 3, 3 },
  { "Floor_RaiseToLowestCeiling", 238, 2, 4, 5 },
  { "Floor_RaiseByValueTxTy", 239, 3, 3, 3 },
  { "Floor_RaiseByTexture", 240, 2, 4, 4 },
  { "Floor_LowerToLowestTxTy", 241, 2, 2, 2 },
  { "Floor_LowerToHighest", 242, 3, 4, 4 },
  { "Exit_Normal", 243, 1, 1, 1 },
  { "Exit_Secret", 244, 1, 1, 1 },
  { "Elevator_RaiseToNearest", 245, 2, 2, 2 },
  { "Elevator_MoveToFloor", 246, 2, 2, 2 },
  { "Elevator_LowerToNearest", 247, 2, 2, 2 },
  { "HealThing", 248, 1, 2, 2 },
  { "Door_CloseWaitOpen", 249, 3, 4, 4 },
  { "Floor_Donut", 250, 3, 3, 3 },
  { "FloorAndCeiling_LowerRaise", 251, 3, 4, 4 },
  { "Ceiling_RaiseToNearest", 252, 2, 3, 3 },
  { "Ceiling_LowerToLowest", 253, 2, 4, 4 },
  { "Ceiling_LowerToFloor", 254, 2, 4, 5 },
  { "Ceiling_CrushRaiseAndStaySilA", 255, 4, 5, 5 },

  { "Floor_LowerToHighestEE", 256, 2, 3, 3 },
  { "Floor_RaiseToLowest", 257, 1, 3, 3 },
  { "Floor_LowerToLowestCeiling", 258, 2, 3, 3 },
  { "Floor_RaiseToCeiling", 259, 2, 4, 5 },
  { "Floor_ToCeilingInstant", 260, 1, 3, 4 },
  { "Floor_LowerByTexture", 261, 2, 3, 3 },
  { "Ceiling_RaiseToHighest", 262, 2, 3, 3 },
  { "Ceiling_ToHighestInstant", 263, 1, 3, 3 },
  { "Ceiling_LowerToNearest", 264, 2, 4, 4 },
  { "Ceiling_RaiseToLowest", 265, 2, 3, 3 },
  { "Ceiling_RaiseToHighestFloor", 266, 2, 3, 3 },
  { "Ceiling_ToFloorInstant", 267, 1, 3, 4 },
  { "Ceiling_RaiseByTexture", 268, 2, 3, 3 },
  { "Ceiling_LowerByTexture", 269, 2, 4, 4 },
  { "Stairs_BuildDownDoom", 270, 5, 5, 5 },
  { "Stairs_BuildUpDoomSync", 271, 4, 4, 4 },
  { "Stairs_BuildDownDoomSync", 272, 4, 4, 4 },
};

zl_linespecial_t* dsda_GetLineSpecialInfo(int special) {
  if (special >= zl_linespecial_max)
    return NULL;

  return &zl_linespecial_spec[special];
}
